@@ -1,5 +1,8 @@
 
 from __future__ import annotations
+import sys
+import json
+import logging
 from abstract_gui import startConsole
 # apiTab_async.py
 # A brand-new non-blocking API console using QNetworkAccessManager (Qt-native async).
@@ -14,9 +17,9 @@ from PyQt6.QtNetwork import QNetworkAccessManager, QNetworkRequest, QNetworkRepl
 from PyQt6.QtWidgets import (
     QApplication, QWidget, QVBoxLayout, QHBoxLayout, QLabel, QLineEdit, QComboBox,
     QPushButton, QTextEdit, QTableWidget, QTableWidgetItem, QAbstractItemView,
-    QMessageBox, QMainWindow, QTableWidgetSelectionRange
+    QMessageBox, QMainWindow, QTableWidgetSelectionRange,QCheckBox
 )
-
+from PyQt6.QtWidgets import QSizePolicy, QSplitter
 # --- Optional: pull user’s helpers if present ---------------------------------
 try:
     # Your project constants/utilities (if available)
@@ -36,20 +39,26 @@ logger = get_logFile(__name__)
 
 # --- Reasonable defaults if your constants aren’t imported ---------------------
 # You can replace these with your PREDEFINED_* from abstract_* if you prefer.
-DEFAULT_BASES: Tuple[Tuple[str, str], ...] = (
-    ("http://127.0.0.1:5000", "Local Flask"),
-    ("http://localhost:8000", "Local Dev"),
-)
-DEFAULT_HEADERS: Tuple[Tuple[str, str], ...] = (
-    ("Accept", "application/json"),
-    ("Content-Type", "application/json"),
-)
+
+
 MIME_TYPES: Dict[str, Dict[str, str]] = {
     "json": {"json": "application/json"},
     "form": {"urlencoded": "application/x-www-form-urlencoded"},
     "text": {"plain": "text/plain"},
 }
-
+DEFAULT_BASES: Tuple[Tuple[str, str], ...] = (
+    ("https://abstractendeavors.com", "abstractendeavors"),
+    ("https://clownworld.biz", "clownworld"),
+    ("https://typicallyoutliers.com", "typicallyoutliers"),
+    ("https://thedailydialectics.com", "thedailydialectics"),
+    ("http://127.0.0.1:5000", "Local Flask"),
+    ("http://localhost:8000", "Local Dev"),
+)
+DEFAULT_HEADERS: Tuple[Tuple[str, str], ...] = (
+    ("Content-Type", "application/json"),
+    ("Accept", "application/json"),
+    ("Authorization", "Bearer "),
+)
 # ------------------------------------------------------------------------------
 # Widget
 # ------------------------------------------------------------------------------
@@ -103,9 +112,10 @@ class apiTab(QWidget):
         self.endpoints_table.setHorizontalHeaderLabels(["Endpoint Path", "Methods"])
         self.endpoints_table.horizontalHeader().setStretchLastSection(True)
         self.endpoints_table.setSelectionBehavior(QAbstractItemView.SelectionBehavior.SelectRows)
-        self.endpoints_table.setFixedHeight(220)
+        self.endpoints_table.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Expanding)
         root.addWidget(self.endpoints_table)
 
+
         # Method override
         mrow = QHBoxLayout()
         mrow.addWidget(QLabel("Override Method:"))
@@ -113,7 +123,26 @@ class apiTab(QWidget):
         self.method_box.addItems(["GET", "POST", "PUT", "PATCH", "DELETE", "OPTIONS"])
         mrow.addWidget(self.method_box)
         root.addLayout(mrow)
+        brow = QHBoxLayout()
+        self.fetch_btn = QPushButton(self._fetch_label(), self)
+        self.prefill_btn = QPushButton("Prefill from Help", self)   # NEW
+        self.help_toggle = QCheckBox("Send help=1 on requests")     # Optional
 
+        self.send_btn = QPushButton("▶ Send", self)
+        self.abort_btn = QPushButton("■ Abort", self)
+        self.abort_btn.setEnabled(False)
+
+        brow.addWidget(self.fetch_btn)
+        brow.addWidget(self.fetch_btn)
+        brow.addWidget(self.prefill_btn)          # NEW
+        brow.addWidget(self.help_toggle)          # Optional
+        brow.addStretch(1)
+        # Buttons
+
+
+        brow.addWidget(self.send_btn)
+        brow.addWidget(self.abort_btn)
+        root.addLayout(brow)
         # Headers
         root.addWidget(QLabel("Headers (check to include; blank key+value inserts new row):"))
         self.headers_table = QTableWidget(len(DEFAULT_HEADERS) + 1, 3, self)
@@ -137,6 +166,13 @@ class apiTab(QWidget):
         self.headers_table.setItem(last, 0, chk)
         self.headers_table.setItem(last, 1, QTableWidgetItem(""))
         self.headers_table.setItem(last, 2, QTableWidgetItem(""))
+        self.headers_table.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Expanding)
+        
+        # Response + Logs inside a splitter so user can resize
+        splitter = QSplitter(self)
+        splitter.setOrientation(Qt.Orientation.Vertical)
+
+
 
         # Body / query params
         root.addWidget(QLabel("Body / Query Params (key → value):"))
@@ -145,33 +181,32 @@ class apiTab(QWidget):
         self.body_table.setFixedHeight(220)
         self.body_table.setItem(0, 0, QTableWidgetItem(""))
         self.body_table.setItem(0, 1, QTableWidgetItem(""))
+        self.body_table.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Expanding)
         root.addWidget(self.body_table)
 
-        # Buttons
-        brow = QHBoxLayout()
-        self.fetch_btn = QPushButton(self._fetch_label(), self)
-        self.send_btn = QPushButton("▶ Send", self)
-        self.abort_btn = QPushButton("■ Abort", self)
-        self.abort_btn.setEnabled(False)
-        brow.addWidget(self.fetch_btn)
-        brow.addStretch(1)
-        brow.addWidget(self.send_btn)
-        brow.addWidget(self.abort_btn)
-        root.addLayout(brow)
+
 
         # Response + Logs
         root.addWidget(QLabel("Response:"))
         self.response_out = QTextEdit(self)
         self.response_out.setReadOnly(True)
-        self.response_out.setFixedHeight(260)
+        self.response_out.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Expanding)
+
         root.addWidget(self.response_out)
 
         root.addWidget(QLabel("Logs:"))
         self.log_out = QTextEdit(self)
         self.log_out.setReadOnly(True)
-        self.log_out.setFixedHeight(160)
+        self.log_out.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Expanding)
         root.addWidget(self.log_out)
 
+        splitter.addWidget(self.response_out)
+        splitter.addWidget(self.log_out)
+        splitter.setStretchFactor(0, 3)  # response 3x bigger
+        splitter.setStretchFactor(1, 1)  # logs smaller
+
+        root.addWidget(QLabel("Response + Logs:"))
+        root.addWidget(splitter)
     def _wire(self):
         self.prefix_in.textChanged.connect(self._on_prefix_changed)
         self.fetch_btn.clicked.connect(self.fetch_endpoints)
@@ -180,11 +215,71 @@ class apiTab(QWidget):
         self.headers_table.cellChanged.connect(self._maybe_add_header_row)
         self.body_table.cellChanged.connect(self._maybe_add_body_row)
         self.detect_btn.clicked.connect(self.detect_prefix)
-
+        self.prefill_btn.clicked.connect(self.prefill_help)  # NEW
     # --------------------------------------------------------------- helpers ---
     def _log(self, msg: str, level: str = "info"):
         self.log_out.append(msg)
         getattr(logger, level, logger.info)(msg)
+    def _clear_body_and_insert(self, rows: list[tuple[str, str]]):
+        self.body_table.blockSignals(True)
+        self.body_table.setRowCount(0)
+        for k, v in rows:
+            r = self.body_table.rowCount()
+            self.body_table.insertRow(r)
+            self.body_table.setItem(r, 0, QTableWidgetItem(k))
+            self.body_table.setItem(r, 1, QTableWidgetItem(v))
+        # trailing blank row
+        r = self.body_table.rowCount()
+        self.body_table.insertRow(r)
+        self.body_table.setItem(r, 0, QTableWidgetItem(""))
+        self.body_table.setItem(r, 1, QTableWidgetItem(""))
+        self.body_table.blockSignals(False)
+    def prefill_help(self):
+        ep = self._selected_endpoint_path()
+        if not ep:
+            QMessageBox.warning(self, "No endpoint", "Select an endpoint row first.")
+            return
+        try:
+            # append ?help=1 to the endpoint path for schema
+            # (your server could also accept POST {"help":true}; adjust if needed)
+            help_ep = ep
+            if "?" in help_ep:
+                help_ep += "&help=1"
+            else:
+                help_ep += "?help=1"
+
+            url = self._build_url(help_ep)
+        except Exception as e:
+            QMessageBox.warning(self, "Invalid URL", str(e))
+            return
+
+        req = QNetworkRequest(QUrl(url))
+        self._log(f"→ GET {url} (prefill help)")
+        reply = self._nam.get(req)
+        self._bind_common(reply, f"GET {url}")
+
+        def after():
+            if reply.error() != QNetworkReply.NetworkError.NoError:
+                return
+            raw = bytes(reply.readAll())
+            try:
+                data = json.loads(raw.decode(errors="replace"))
+            except Exception:
+                return
+            if isinstance(data, dict):
+                # show doc for context (optional)
+                self._maybe_show_doc(data)
+                rows = self._rows_from_help_schema(data)
+                if rows:
+                    self._clear_body_and_insert(rows)
+                    self._log("✓ Prefilled params from help")
+        reply.finished.connect(after)
+    def _selected_endpoint_path(self) -> str | None:
+        sel = self.endpoints_table.selectionModel().selectedRows()
+        if not sel:
+            return None
+        path = self.endpoints_table.item(sel[0].row(), 0)
+        return path.text().strip() if path else None
 
     def _fetch_label(self) -> str:
         p = (self.prefix_in.text().strip() or "/api")
@@ -215,7 +310,46 @@ class apiTab(QWidget):
             self.headers_table.setItem(last + 1, 1, QTableWidgetItem(""))
             self.headers_table.setItem(last + 1, 2, QTableWidgetItem(""))
             self.headers_table.blockSignals(False)
+    def _maybe_show_doc(self, obj: dict):
+        doc = None
+        if "doc" in obj and isinstance(obj["doc"], str):
+            doc = obj["doc"]
+        elif len(obj) == 1 and isinstance(next(iter(obj.values())), dict):
+            inner = next(iter(obj.values()))
+            doc = inner.get("doc")
+        if doc:
+            self.response_out.setPlainText(doc.strip())
+    def _rows_from_help_schema(self, obj: dict) -> list[tuple[str, str]]:
+        """
+        Accepts formats like:
+        {
+          "analyze_visible_surface": {
+            "doc": "...",
+            "params": [{"name":"altitude_step","default":200.0}, ...]
+          }
+        }
+        or directly {"doc": ..., "params":[...]}
+        """
+        # unwrap if top-level is {funcName: {...}}
+        if "params" not in obj and len(obj) == 1 and isinstance(next(iter(obj.values())), dict):
+            obj = next(iter(obj.values()))
 
+        rows: list[tuple[str, str]] = []
+        params = obj.get("params") or []
+        for p in params:
+            name = str(p.get("name", "")).strip()
+            if not name:
+                continue
+            default = p.get("default", "")
+            # render default as string safely
+            if isinstance(default, (dict, list)):
+                val = json.dumps(default)
+            elif default is None:
+                val = ""
+            else:
+                val = str(default)
+            rows.append((name, val))
+        return rows
     def _maybe_add_body_row(self, row: int, _col: int):
         last = self.body_table.rowCount() - 1
         key_item = self.body_table.item(row, 0)
@@ -277,39 +411,62 @@ class apiTab(QWidget):
         self.abort_btn.setEnabled(False)
         QApplication.restoreOverrideCursor()
 
-    def _bind_common(self, reply: QNetworkReply, label: str):
+    def _bind_common(self, reply: QNetworkReply, label: str, after=None):
+        """
+        Bind common handlers; optionally run `after(parsed_json)` exactly once
+        after the reply is fully read. `after` receives:
+           - parsed_obj (dict/list/None) from JSON parse (None if not JSON)
+           - raw_text (str) full decoded payload
+        """
         self._inflight = reply
-        reply.finished.connect(lambda: self._on_finished(reply, label))
+        # stash the optional callback on the reply itself
+        reply.setProperty(b"_after_cb", after)
+        reply.setProperty(b"_label", label)
+        reply.finished.connect(lambda: self._on_finished(reply))
         reply.errorOccurred.connect(lambda _err: self._on_error(reply, label))
         self._start_timeout()
-
     def _on_timeout(self):
-        if self._inflight:
-            self._log("⏳ Request timed out; aborting.", "warning")
-            self._inflight.abort()
-        self._stop_timeout()
+            if self._inflight:
+                self._log("⏳ Request timed out; aborting.", "warning")
+                self._inflight.abort()
+            self._stop_timeout()
 
     def abort_request(self):
         if self._inflight:
             self._log("■ Abort requested by user.", "warning")
             self._inflight.abort()
 
-    def _on_finished(self, reply: QNetworkReply, label: str):
+    def _on_finished(self, reply: QNetworkReply):
         self._stop_timeout()
+        label = reply.property(b"_label") or ""
+        after = reply.property(b"_after_cb")
+
         if reply.isFinished() and reply.error() == QNetworkReply.NetworkError.NoError:
+            # read payload ONCE
             data = bytes(reply.readAll())
             text = data.decode(errors="replace")
-            # try pretty JSON
+
+            # try parse JSON (don’t throw)
+            parsed = None
             try:
-                obj = json.loads(text)
-                text = json.dumps(obj, indent=2)
+                parsed = json.loads(text)
+                pretty = json.dumps(parsed, indent=2)
+                self.response_out.setPlainText(pretty)
             except Exception:
-                pass
-            self.response_out.setPlainText(text)
+                self.response_out.setPlainText(text)
+
             self._log(f"✔ {label}")
+
+            # if a callback was registered, hand over parsed + raw text
+            if callable(after):
+                try:
+                    after(parsed, text)
+                except Exception as e:
+                    self._log(f"after() callback error: {e}", "error")
         reply.deleteLater()
         self._inflight = None
 
+
     def _on_error(self, reply: QNetworkReply, label: str):
         self._stop_timeout()
         err = reply.error()
@@ -318,10 +475,8 @@ class apiTab(QWidget):
         self._log(f"✖ {label} — {err}: {msg}", "error")
         reply.deleteLater()
         self._inflight = None
-
-    # ------------------------------------------------------------- actions ----
     def fetch_endpoints(self):
-        """GET {base}{prefix}/endpoints -> list[[path, methods], ...]"""
+        """GET {base}{prefix}/endpoints -> list[[path, methods], ...] and populate table."""
         try:
             url = self._build_url("/endpoints")
         except Exception as e:
@@ -331,42 +486,81 @@ class apiTab(QWidget):
         req = QNetworkRequest(QUrl(url))
         self._log(f"→ GET {url}")
         reply = self._nam.get(req)
-        self._bind_common(reply, f"GET {url}")
 
-        # Also populate table when finished (if parseable)
-        def after():
-            if reply.error() != QNetworkReply.NetworkError.NoError:
-                return
-            raw = bytes(reply.readAll())
-            try:
-                data = json.loads(raw.decode(errors="replace"))
-            except Exception:
-                return
+        # define how to populate when the JSON arrives
+        def _after(parsed, _raw_text):
+            if isinstance(parsed, list):
+                self._populate_endpoints(parsed)
+            elif isinstance(parsed, dict) and "endpoints" in parsed and isinstance(parsed["endpoints"], list):
+                self._populate_endpoints(parsed["endpoints"])
+            # else: leave the response text as-is, no table update
+
+        # bind with after-callback so body is read only once
+        self._bind_common(reply, f"GET {url}", after=_after)
+
+    # ------------------------------------------------------------- actions ----
+    # ── network actions (sync via abstract_apis) ─────────────────────────
+    def fetch_remote_endpoints(self):
+        base = self.base_combo.currentText().rstrip('/')
+        url = f"{base}/api/endpoints"
+        self.log_output.clear()
+        logging.info(f"Fetching remote endpoints from {url}")
+        try:
+            data = getRequest(url=url)
             if isinstance(data, list):
                 self._populate_endpoints(data)
-
-        reply.finished.connect(after)
+                logging.info("✔ Remote endpoints loaded")
+            else:
+                logging.warning("/api/endpoints returned non-list, ignoring")
+        except Exception as e:
+            logging.error(f"Failed to fetch endpoints: {e}")
+            QMessageBox.warning(self, "Fetch Error", str(e))
 
     def _populate_endpoints(self, lst):
-        self.endpoints_table.setRowCount(0)
+        self.endpoints_table.clearContents()
+        self.endpoints_table.setRowCount(len(lst))
         for i, item in enumerate(lst):
-            # Accept either ("path", "METHODS") or {"path":..., "methods":...}
             if isinstance(item, (list, tuple)) and len(item) >= 2:
                 path, methods = item[0], item[1]
             elif isinstance(item, dict):
                 path, methods = item.get("path", ""), item.get("methods", "")
             else:
-                continue
-            r = self.endpoints_table.rowCount()
-            self.endpoints_table.insertRow(r)
-            self.endpoints_table.setItem(r, 0, QTableWidgetItem(str(path)))
-            self.endpoints_table.setItem(r, 1, QTableWidgetItem(str(methods)))
+                path, methods = str(item), ""
+            self.endpoints_table.setItem(i, 0, QTableWidgetItem(path))
+            self.endpoints_table.setItem(i, 1, QTableWidgetItem(methods))
 
-        # Select first row automatically to speed testing
-        if self.endpoints_table.rowCount():
-            self.endpoints_table.setRangeSelected(
-                QTableWidgetSelectionRange(0, 0, 0, 1), True
-            )
+    def on_endpoint_selected(self, row: int, _col: int):
+        ep = self.endpoints_table.item(row, 0).text()
+        cfg = self.config_cache.get(ep, {})
+        # restore method
+        if 'method' in cfg:
+            self.method_box.setCurrentText(cfg['method'])
+        # restore headers
+        for r in range(self.headers_table.rowCount()):
+            chk = self.headers_table.item(r, 0)
+            key_item = self.headers_table.item(r, 1)
+            if not chk or not key_item:
+                continue
+            k = key_item.text()
+            if k in cfg.get('headers', {}):
+                chk.setCheckState(Qt.CheckState.Checked)
+                self.headers_table.setItem(r, 2, QTableWidgetItem(cfg['headers'][k]))
+            else:
+                chk.setCheckState(Qt.CheckState.Unchecked)
+        # restore params
+        self.body_table.blockSignals(True)
+        self.body_table.setRowCount(0)
+        for k, v in cfg.get('params', {}).items():
+            idx = self.body_table.rowCount()
+            self.body_table.insertRow(idx)
+            self.body_table.setItem(idx, 0, QTableWidgetItem(k))
+            self.body_table.setItem(idx, 1, QTableWidgetItem(v))
+        # ensure one blank editable row
+        idx = self.body_table.rowCount()
+        self.body_table.insertRow(idx)
+        self.body_table.setItem(idx, 0, QTableWidgetItem(""))
+        self.body_table.setItem(idx, 1, QTableWidgetItem(""))
+        self.body_table.blockSignals(False)
 
     def send_request(self):
         sel = self.endpoints_table.selectionModel().selectedRows()
@@ -411,7 +605,9 @@ class apiTab(QWidget):
                 if kv and not ctype:
                     req.setRawHeader(b"Content-Type", b"application/json")
                     body_bytes = QByteArray(json.dumps(kv).encode())
-
+        if self.help_toggle.isChecked():
+            kv = dict(kv)  # copy
+            kv["help"] = 1
         # Dispatch
         if method == "GET":
             # For GET, append query string
@@ -423,7 +619,9 @@ class apiTab(QWidget):
                 req.setUrl(u)
             reply = self._nam.get(req)
         elif method == "POST":
-            reply = self._nam.post(req, body_bytes or QByteArray())
+            req = QNetworkRequest(QUrl(self._build_url(ep)))
+            req.setHeader(QNetworkRequest.KnownHeaders.ContentTypeHeader, "application/json")
+            reply = self._nam.post(req, QByteArray(json.dumps({"help": True}).encode()))
         elif method == "PUT":
             reply = self._nam.put(req, body_bytes or QByteArray())
         elif method == "PATCH":
@@ -491,4 +689,3 @@ class apiTab(QWidget):
         event.accept()
 
 
-
